import java.util.Date;

public class ImmutableTest
{
   public static void main( String[] args )
   {
      final Date start = new Date();
      final Date end   = new Date();
      final Period1 p  = new Period1( start, end ); // Period1 durch Period2 ersetzen
      System.out.println( p );

      end.setTime( 1234567890 );
      System.out.println( p );

      p.getStart().setTime( 12345678900L );
      System.out.println( p );
   }
}

final class Period1 // Mutable
{
   private final Date start;
   private final Date end;

   public Period1( Date start, Date end )
   {
      if( start.compareTo( end ) > 0 )
         throw new IllegalArgumentException( start + " after " + end );
      this.start = start;
      this.end   = end;
   }

   public Date getStart() { return start; }
   public Date getEnd()   { return end;   }

   @Override
   public String toString() { return start + " - " + end; }
}

final class Period2 // Immutable
{
   private final Date start;
   private final Date end;

   public Period2( Date start, Date end )
   {
      if( start.compareTo( end ) > 0 )
         throw new IllegalArgumentException( start + " after " + end );
      this.start = new Date( start.getTime() );
      this.end   = new Date( end.getTime() );
   }

   public Date getStart() { return new Date( start.getTime() ); }
   public Date getEnd()   { return new Date( end.getTime() );   }

   @Override
   public String toString() { return start + " - " + end; }
}
