import java.util.concurrent.*;

/**
 * Nach einer Vorlage von:
 * @author Brian Goetz und Tim Peierls
 */
public class Memoizer<A,V> implements Computable<A,V>
{
   private   final ConcurrentMap<A,Future<V>> cache = new ConcurrentHashMap<A,Future<V>>();
   protected final Computable<A,V> c;

   public Memoizer( Computable<A,V> c )
   {
      this.c = c;
   }

   @Override
   public V compute( final A arg )
   {
      while( true ) {
         Future<V> f = cache.get( arg );
         if( f == null ) {
            Callable<V> eval = new Callable<V>() {
               @Override public V call() throws InterruptedException {
                  return c.compute( arg );
               }
            };
            FutureTask<V> ft = new FutureTask<V>( eval );
            f = cache.putIfAbsent( arg, ft );
            if( f == null ) {
               f = ft;
               ft.run();
            }
         }
         try {
            return f.get();
         } catch( CancellationException ex ) {
            cache.remove( arg, f );
         } catch( Exception ex ) {
            cache.remove( arg, f );
            throw new RuntimeException( ex.getCause() );
         }
      }
   }
}

interface Computable<A,V>
{
   V compute( A arg ) throws InterruptedException;
}
