import java.util.Random;
import java.util.concurrent.TimeUnit;

public class MemoizerTest
{
   public static void main( String[] args )
   {
      Computable<String,Double> c = new LangwierigeBerechnung();
      starteThreads( "Ohne Cache",                 c );
      starteThreads( "Synchronized Memoizer",      new SynchronizedMemoizer<String,Double>( c ) );
      starteThreads( "ConcurrentHashMap Memoizer", new ConcurrentHashMapMemoizer<String,Double>( c ) );
      starteThreads( "Optimierter Memoizer",       new Memoizer<String,Double>( c ) );
   }

   static void starteThreads( String s, Computable<String,Double> c )
   {
      for( int i=0; i<5; i++ ) { // eventuell die Anzahl der Threads aendern
         new Thread( (new MemoizerRunnable( s, c )) ).start();
      }
   }
}

class LangwierigeBerechnung implements Computable<String,Double>
{
   @Override
   public Double compute( String arg ) throws InterruptedException
   {
      TimeUnit.MILLISECONDS.sleep( 10 ); // simuliert z.B. Datenbankabfrage
      return new Double( arg );
   }
}

class MemoizerRunnable implements Runnable
{
   static final int[]        ARGARR = { 1, 42, 7, 1629, 4711, 1984, 999, 2, 3, 11 };
   String                    s;
   Computable<String,Double> c;

   MemoizerRunnable( String s, Computable<String,Double> c )
   {
      this.s = s;
      this.c = c;
   }

   @Override
   public void run()
   {
      long startZeit = System.nanoTime();
      int j = (new Random()).nextInt( 10 );
      for( int i=0; i<20; i++ ) { // eventuell die Lese-Anzahl aendern
         try {
            TimeUnit.MILLISECONDS.sleep( 1 );
            c.compute( "" + ARGARR[(i + j) % ARGARR.length] );
         } catch( InterruptedException ex ) {
            throw new RuntimeException( ex );
         }
      }
      System.out.println( s + ": " + ((System.nanoTime() - startZeit) / 1000 / 1000.) + " ms" );
   }
}
