import java.util.List;
import java.util.concurrent.*;

/**
 * Parallele Abarbeitung vieler Tasks.
 * Aufruf ueber:
 * (new TasksConcurrentExecutor<InputType,ResultType>( taskList, threadCount, timeoutSeconds )).start();
 * Die taskList muss die TaskDescription-Objekte enthalten.
 * Die Ergebnisse befinden sich anschliessend in den Result-Objekten in den Tasks in der taskList.
 */
public class TasksConcurrentExecutor<A,R>
{
   private List<TaskDescription<A,R>> taskList;
   private int                        threadCount;
   private long                       timeoutSeconds;

   public TasksConcurrentExecutor( List<TaskDescription<A,R>> taskList, int threadCount, long timeoutSeconds )
   {
      this.taskList       = taskList;
      this.threadCount    = threadCount;
      this.timeoutSeconds = timeoutSeconds;
   }

   public void start()
   {
      ExecutorService threadPool = Executors.newFixedThreadPool( threadCount );
      for( TaskDescription<A,R> td : taskList ) {
         threadPool.execute( new TaskRunnable( td ) );
      }
      threadPool.shutdown();
      try {
         if( !threadPool.awaitTermination( timeoutSeconds, TimeUnit.SECONDS ) ) {
            threadPool.shutdownNow();
            throw new RuntimeException( "Timeout (" + timeoutSeconds + " Sekunden) ueberschritten." );
         }
      } catch( InterruptedException ex ) {
         throw new RuntimeException( ex );
      }
   }

   public class TaskRunnable implements Runnable
   {
      TaskDescription<A,R> td;

      public TaskRunnable( TaskDescription<A,R> td )
      {
         this.td = td;
      }

      @Override public void run()
      {
         // System.out.println( Thread.currentThread().getName() + ": arg=" + td.arg );
         td.result = td.computable.compute( td.arg );
         // System.out.println( Thread.currentThread().getName() + ": result=" + td.result );
      }
   }
}
