import java.util.*;
import java.util.concurrent.TimeUnit;
import org.junit.*;

/** Test fuer @see TasksConcurrentExecutor */
public class TasksConcurrentExecutorTest
{
   /** Teste die Parallelitaet */
   @Test
   public void testTasksConcurrentExecutorParallel()
   {
      testTasksConcurrentExecutorParallelIntern( 1, 20, 340 );
      testTasksConcurrentExecutorParallelIntern( 8, 20,  60 );
   }

   private static void testTasksConcurrentExecutorParallelIntern( int threadCount, int taskCount, int maxDauerMs )
   {
      // Die Beispiel-Tasks haben ein Integer-Input-Argument und ein String-Result:
      TaskDescription.Computable<Integer,String> testComputable = new TaskDescription.Computable<Integer,String>() {
         @Override public String compute( Integer arg ) {
            // Hier die auszufuehrende Task-Arbeitsmethode implementieren ...:
            try { TimeUnit.MILLISECONDS.sleep( arg.intValue() ); } catch( InterruptedException ex ) { /* ok */ }
            return "" + arg;
         }
      };
      List<TaskDescription<Integer,String>> taskList = new ArrayList<TaskDescription<Integer,String>>();

      // Erzeuge die auszufuehrenden Task-Objekte und sammle in der taskList:
      for( int i = 0; i < taskCount; i++ )
         taskList.add( new TaskDescription<Integer,String>( testComputable, new Integer( (new Random()).nextInt( 10 ) ) ) );

      // Fuehre die Tasks parallelisiert aus:
      long startZeit = System.nanoTime();
      (new TasksConcurrentExecutor<Integer,String>( taskList, threadCount, 1 )).start();
      long dauerMs = ((System.nanoTime() - startZeit) / 1000000);

      Assert.assertTrue( taskList.size() + " Tasks in " + threadCount + " Threads: Dauer ist " + dauerMs + " ms, "
                         + "erlaubt sind maximal " + maxDauerMs + " ms.", dauerMs <= maxDauerMs );
   }

   /** Teste die Timeout-Funktionalitaet */
   @Test
   public void testTasksConcurrentExecutorTimeout()
   {
      TaskDescription.Computable<Integer,String> testComputable = new TaskDescription.Computable<Integer,String>() {
         @Override public String compute( Integer arg ) {
            try { TimeUnit.MILLISECONDS.sleep( arg.intValue() ); } catch( InterruptedException ex ) { /* ok */ }
            return "" + arg;
         }
      };
      List<TaskDescription<Integer,String>> taskList = new ArrayList<TaskDescription<Integer,String>>();

      // Test mit 100 Millisekunden (kleiner als Timeout von 1 Sekunde):
      TaskDescription<Integer,String> td = new TaskDescription<Integer,String>( testComputable, Integer.valueOf( 100 ) );
      taskList.add( td );
      (new TasksConcurrentExecutor<Integer,String>( taskList, 1, 1 )).start();
      Assert.assertEquals( "100", td.result );

      // Test mit 1100 Millisekunden (groesser als Timeout von 1 Sekunde):
      taskList.clear();
      taskList.add( new TaskDescription<Integer,String>( testComputable, Integer.valueOf( 1100 ) ) );
      try {
         (new TasksConcurrentExecutor<Integer,String>( taskList, 1, 1 )).start();
         Assert.fail( "Nach Timeout-Ueberschreitung muss Exception geworfen werden." );
      } catch( RuntimeException ex ) {
         Assert.assertEquals( "Timeout (1 Sekunden) ueberschritten.", ex.getMessage() );
      }
   }
}
